
package com.bweb.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.domain.ForbbidenTime;
import com.alipay.api.domain.OfferObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.pay.util.sdk.WXPayConstants;
import com.hai.common.utils.*;
import com.hai.config.*;
import com.hai.entity.*;
import com.hai.model.OrderRefundModel;
import com.hai.model.ResponseData;
import com.hai.order.service.OrderRefundService;
import com.hai.order.service.OrderService;
import com.hai.order.type.OrderPayType;
import com.hai.order.type.OrderRefundOpUserType;
import com.hai.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/3/26 23:08
 */
@Controller
@RequestMapping(value = "/test")
@Api(value = "订单接口")
public class HighTestController {

    private static Logger log = LoggerFactory.getLogger(HighTestController.class);


    @Resource
    private OutRechargeOrderService outRechargeOrderService;

    @Resource
    private OrderService orderService;

    @Resource
    private HighOrderService highOrderService;

    @Resource
    private OrderRefundService orderRefundService;

    @RequestMapping(value="/resolveResponse",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "解析")
    public ResponseData resolveResponse(
            @RequestParam(name = "data", required = false) String data
    ) {
        try {
            JSONObject cardInfoObject = HuiLianTongUnionCardConfig.resolveResponse(data);
            return ResponseMsgUtil.success(cardInfoObject);

        } catch (Exception e) {
            log.error("getUserByTelephone",e);
            return ResponseMsgUtil.exception(e);
        }
    }



    @RequestMapping(value = "/getHuiLianTongCardConsume", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询汇联通工会卡消费记录")
    public ResponseData getHuiLianTongCardConsume(@RequestParam(name = "businessType", required = true) String businessType,
                                                  @RequestParam(name = "cardNo", required = true) String cardNo,
                                                  @RequestParam(name = "sdate", required = true) Long sdate,
                                                  @RequestParam(name = "edate", required = true) Long edate,
                                                  @RequestParam(name = "pageNum", required = true) Integer pageNum,
                                                  @RequestParam(name = "pageSize", required = true) Integer pageSize,
                                                  HttpServletRequest request) {
        try {


            JSONObject consumptionRecord = HuiLianTongUnionCardConfig.queryConsumptionRecordByBusiness(businessType, cardNo, sdate, edate, pageNum, pageSize);
            if (StringUtils.isBlank(consumptionRecord.getString("data"))) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQUEST_ERROR, "");
            }

            JSONArray array = HuiLianTongUnionCardConfig.resolveResponse(consumptionRecord.getString("data")).getJSONArray("records");

            return ResponseMsgUtil.success(array);

        } catch (Exception e) {
            log.error("HighUserCardController --> getHuiLianTongCardConsume() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value = "/insertV2", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "支付")
    public ResponseData insertV2() {
        try {

            Map<String , Object> map = new HashMap<>();
            map.put("timetable" , new Date().getTime());
            map.put("apiKey" , "9A838BEBC60712775E119366DB804C43");
            map.put("orderNo" , "20220329182036435619");
            map.put("count" , "1");
            map.put("couTypeCode" , "test");
            map.put("phone" , "15206071196");
            map.put("merchId" , "20220670800");
            map.put("sign" , WxUtils.generateSignApi(map, MapUtils.getString(map, "apiKey"), WXPayConstants.SignType.MD5));

            return ResponseMsgUtil.success(HttpsUtils.doPost("https://hsgcs.dctpay.com/v1/gzSinopec/couJointDist" , map));
        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }



    @RequestMapping(value = "/phoneBillOrderError", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "话费订单错误")
    public ResponseData phoneBillOrderError() {
        List<HighOrder> highOrders = orderService.phoneErrorOrder();
        for (HighOrder order: highOrders) {

            try {

                OutRechargeOrder outRechargeOrder = outRechargeOrderService.findByOrderNo(order.getOrderNo());

                OutRechargeChildOrder childOrder = new OutRechargeChildOrder();

                if (outRechargeOrder.getOutRechargeChildOrders().size() > 0) {
                    childOrder = outRechargeOrder.getOutRechargeChildOrders().get(0);
                }
                outRechargeOrderService.rechargeOrderHandle(outRechargeOrder , childOrder);


            } catch (Exception e) {
                log.error("HighCouponSchedule --> expiredCoupon() error!", e);
            }
        }

        return ResponseMsgUtil.success("操作成功");

    }


    @RequestMapping(value = "/checkOrderByOrderNo", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据订单号话费查询是否可以退款")
    public ResponseData checkOrderByOrderNo(@RequestParam(name = "orderNo", required = true) String orderNo) {
        try {

            OutRechargeOrder outRechargeOrder = outRechargeOrderService.findByOrderNo(orderNo);

            OutRechargeChildOrder childOrder = new OutRechargeChildOrder();

            if (outRechargeOrder.getOutRechargeChildOrders().size() > 0) {
                childOrder = outRechargeOrder.getOutRechargeChildOrders().get(0);
            }
            outRechargeOrderService.rechargeOrderHandle(outRechargeOrder , childOrder);


            return ResponseMsgUtil.success("退款成功");
        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value = "/orderRefund", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "订单退款")
    public ResponseData orderRefund() {
        try {
            Calendar createTimeS = Calendar.getInstance();
            createTimeS.set(Calendar.DATE, 17);
            createTimeS.set(Calendar.HOUR_OF_DAY , 00);
            createTimeS.set(Calendar.MINUTE , 00);
            createTimeS.set(Calendar.SECOND  , 00);

            Calendar createTimeE = Calendar.getInstance();
            createTimeE.set(Calendar.DATE, 20);
            createTimeE.set(Calendar.HOUR_OF_DAY , 15);
            createTimeE.set(Calendar.MINUTE , 25);
            createTimeE.set(Calendar.SECOND  , 00);

            Map<String,Object> param = new HashMap<>();
            param.put("createTimeS", createTimeS.getTimeInMillis());
            param.put("createTimeE", createTimeE.getTimeInMillis());
            List<HighOrderRefund> orderRefundList = orderRefundService.getRefundOrderList(param);
            for (HighOrderRefund orderRefund : orderRefundList) {
                try {
                    orderRefundService.refundPrice(
                            orderRefund.getRefundOrderNo(),
                            orderService.getOrderDetailByNo(orderRefund.getOrderNo()),
                            OrderPayType.getDataByType(orderRefund.getPayType()),
                            orderRefund.getRefundPrice(),
                            orderRefund.getRefundIntegral()
                    );
                    orderRefund.setExt1("true");
                } catch (Exception e) {
                    orderRefund.setExt1("false");
                } finally {
                    orderRefundService.update(orderRefund);
                }
            }

            return ResponseMsgUtil.success("退款成功");

        } catch (Exception e) {
            log.error("HighOrderController --> getBackendToken() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    public String orderTitle(OutRechargeOrder outRechargeOrder) {
        // 1 待支付  2 已支付  3.已完成  4. 已退款  5.已取消
        //  101.待支付 102.已支付 100.已完成 104.已取消 105.已退款
        switch (outRechargeOrder.getOperatorType()) {
            case 1:
                return outRechargeOrder.getRechargeContent() + "电信话费充值" + outRechargeOrder.getRechargePrice() + "元";
            case 2:
                return outRechargeOrder.getRechargeContent() + "移动话费充值" + outRechargeOrder.getRechargePrice() + "元";
            case 3:
                return outRechargeOrder.getRechargeContent() + "联通话费充值" + outRechargeOrder.getRechargePrice() + "元";
            default:
                return null;
        }
    }
}


